const coral = @import("coral");

pub const CheckError = error {
	CheckFailed
};

pub const CompileError = coral.io.AllocationError || RuntimeError || error {
	UnexpectedEnd,
	UnexpectedToken,
	UndefinedLocal,
};

pub const Float = f32;

pub const Integer = i32;

pub const Object = u32;

pub const Primitive = enum {
	nil,
	false,
	true,
	float,
	integer,
	object,
};

pub const Ref = union (Primitive) {
	nil,
	false,
	true,
	float: Float,
	integer: Integer,
	object: Object,
};

pub const RuntimeError = coral.io.AllocationError || CheckError || error {
	BadSyntax,
};

pub fn SmallStack(comptime Element: type, comptime default: Element) type {
	const maximum = 255;

	return struct {
		buffer: [maximum]Element = [_]Element{default} ** maximum,
		count: u8 = 0,

		const Self = @This();

		fn peek(self: Self) ?Element {
			if (self.count == 0) return null;

			return self.buffer[self.count - 1];
		}

		fn pop(self: *Self) ?Element {
			if (self.count == 0) return null;

			self.count -= 1;

			return self.buffer[self.count];
		}

		fn push(self: *Self, element: Element) !void {
			if (self.count == maximum) return error.OutOfMemory;

			self.buffer[self.count] = element;
			self.count += 1;
		}
	};
}

pub const Val = union (Primitive) {
	nil,
	false,
	true,
	float: Float,
	integer: Integer,
	object: Object,

	pub fn as_ref(self: *const Val) Ref {
		return switch (self.*) {
			.nil => .nil,
			.false => .false,
			.true => .true,
			.float => .{.float = self.float},
			.integer => .{.integer = self.integer},
			.object => .{.object = self.object},
		};
	}
};
